from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_GET_DLL, SDL_TTF_DLL, SDL_DLL

from .SDL_pixels import SDL_Color
from .SDL_surface import SDL_Surface
from .SDL_iostream import SDL_IOStream
from .SDL_version import SDL_VERSIONNUM

SDL_SET_CURRENT_DLL(SDL_TTF_DLL)

SDL_TTF_MAJOR_VERSION = 3
SDL_TTF_MINOR_VERSION = 0
SDL_TTF_MICRO_VERSION = 0

SDL_TTF_VERSION = \
    SDL_VERSIONNUM(SDL_TTF_MAJOR_VERSION, SDL_TTF_MINOR_VERSION, SDL_TTF_MICRO_VERSION)

SDL_TTF_VERSION_ATLEAST = lambda x, y, z: \
    (SDL_TTF_MAJOR_VERSION >= x) and \
    (SDL_TTF_MAJOR_VERSION > x or SDL_TTF_MINOR_VERSION >= y) and \
    (SDL_TTF_MAJOR_VERSION > x or SDL_TTF_MINOR_VERSION > y or SDL_TTF_MICRO_VERSION >= z)

SDL_FUNC("TTF_Version", ctypes.c_int)
SDL_FUNC("TTF_GetFreeTypeVersion", None, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_GetHarfBuzzVersion", None, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

UNICODE_BOM_NATIVE = 0xFEFF
UNICODE_BOM_SWAPPED = 0xFFFE

SDL_FUNC("TTF_ByteSwappedUNICODE", None, ctypes.c_bool)

class TTF_Font(ctypes.Structure):
    ...

SDL_FUNC("TTF_Init", ctypes.c_int)
SDL_FUNC("TTF_OpenFont", ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("TTF_OpenFontIndex", ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int, ctypes.c_long)
SDL_FUNC("TTF_OpenFontIO", ctypes.POINTER(TTF_Font), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int)
SDL_FUNC("TTF_OpenFontIndexIO", ctypes.POINTER(TTF_Font), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int, ctypes.c_long)
SDL_FUNC("TTF_OpenFontDPI", ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int, ctypes.c_uint, ctypes.c_uint)
SDL_FUNC("TTF_OpenFontIndexDPI", ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int, ctypes.c_long, ctypes.c_uint, ctypes.c_uint)
SDL_FUNC("TTF_OpenFontDPIIO", ctypes.POINTER(TTF_Font), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int, ctypes.c_uint, ctypes.c_uint)
SDL_FUNC("TTF_OpenFontIndexDPIIO", ctypes.POINTER(TTF_Font), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int, ctypes.c_long, ctypes.c_uint, ctypes.c_uint)
SDL_FUNC("TTF_SetFontSize", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_int)
SDL_FUNC("TTF_SetFontSizeDPI", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_int, ctypes.c_uint, ctypes.c_uint)

TTF_STYLE_NORMAL = 0x00
TTF_STYLE_BOLD = 0x01
TTF_STYLE_ITALIC = 0x02
TTF_STYLE_UNDERLINE = 0x04
TTF_STYLE_STRIKETHROUGH = 0x08

SDL_FUNC("TTF_GetFontStyle", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_SetFontStyle", None, ctypes.POINTER(TTF_Font), ctypes.c_int)
SDL_FUNC("TTF_GetFontOutline", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_SetFontOutline", None, ctypes.POINTER(TTF_Font), ctypes.c_int)

TTF_HINTING_NORMAL = 0
TTF_HINTING_LIGHT = 1
TTF_HINTING_MONO = 2
TTF_HINTING_NONE = 3
TTF_HINTING_LIGHT_SUBPIXEL = 4

SDL_FUNC("TTF_GetFontHinting", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_SetFontHinting", None, ctypes.POINTER(TTF_Font), ctypes.c_int)

TTF_WRAPPED_ALIGN_LEFT = 0
TTF_WRAPPED_ALIGN_CENTER = 1
TTF_WRAPPED_ALIGN_RIGHT = 2

SDL_FUNC("TTF_GetFontWrappedAlign", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_SetFontWrappedAlign", None, ctypes.POINTER(TTF_Font), ctypes.c_int)

SDL_FUNC("TTF_FontHeight", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontAscent", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontDescent", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontLineSkip", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_GetFontKerning", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_SetFontKerning", None, ctypes.POINTER(TTF_Font), ctypes.c_int)
SDL_FUNC("TTF_FontFaces", ctypes.c_long, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontFaceIsFixedWidth", ctypes.c_int, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontFaceFamilyName", ctypes.c_char_p, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_FontFaceStyleName", ctypes.c_char_p, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_GlyphIsProvided", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint16)
SDL_FUNC("TTF_GlyphIsProvided32", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint32)
SDL_FUNC("TTF_GlyphMetrics", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint16, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_GlyphMetrics32", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint32, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_SizeText", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_SizeUTF8", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_SizeUNICODE", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_MeasureText", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_MeasureUTF8", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p, ctypes.c_int, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_MeasureUNICODE", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), ctypes.c_int, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("TTF_RenderText_Solid", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color)
SDL_FUNC("TTF_RenderUTF8_Solid", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color)
SDL_FUNC("TTF_RenderUNICODE_Solid", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color)
SDL_FUNC("TTF_RenderText_Solid_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUTF8_Solid_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUNICODE_Solid_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderGlyph_Solid", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint16, SDL_Color)
SDL_FUNC("TTF_RenderGlyph32_Solid", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint32, SDL_Color)
SDL_FUNC("TTF_RenderText_Shaded", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderUTF8_Shaded", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderUNICODE_Shaded", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderText_Shaded_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUTF8_Shaded_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUNICODE_Shaded_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderGlyph_Shaded", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint16, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderGlyph32_Shaded", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint32, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderText_Blended", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color)
SDL_FUNC("TTF_RenderUTF8_Blended", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color)
SDL_FUNC("TTF_RenderUNICODE_Blended", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color)
SDL_FUNC("TTF_RenderText_Blended_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUTF8_Blended_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUNICODE_Blended_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderGlyph_Blended", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint16, SDL_Color)
SDL_FUNC("TTF_RenderGlyph32_Blended", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint32, SDL_Color)
SDL_FUNC("TTF_RenderText_LCD", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderUTF8_LCD", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderUNICODE_LCD", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderText_LCD_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUTF8_LCD_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_char_p, SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderUNICODE_LCD_Wrapped", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.POINTER(ctypes.c_uint16), SDL_Color, SDL_Color, ctypes.c_uint32)
SDL_FUNC("TTF_RenderGlyph_LCD", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint16, SDL_Color, SDL_Color)
SDL_FUNC("TTF_RenderGlyph32_LCD", ctypes.POINTER(SDL_Surface), ctypes.POINTER(TTF_Font), ctypes.c_uint32, SDL_Color)

TTF_RenderText = lambda font, text, fg, bg: \
    SDL_GET_DLL(SDL_TTF_DLL).TTF_RenderText_Shaded(font, text, fg, bg)

TTF_RenderUTF8 = lambda font, text, fg, bg: \
    SDL_GET_DLL(SDL_TTF_DLL).TTF_RenderUTF8_Shaded(font, text, fg, bg)

TTF_RenderUNICODE = lambda font, text, fg, bg: \
    SDL_GET_DLL(SDL_TTF_DLL).TTF_RenderUNICODE_Shaded(font, text, fg, bg)

SDL_FUNC("TTF_CloseFont", None, ctypes.POINTER(TTF_Font))
SDL_FUNC("TTF_Quit", None)
SDL_FUNC("TTF_WasInit", ctypes.c_int)
SDL_FUNC("TTF_GetFontKerningSizeGlyphs", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint16, ctypes.c_uint16)
SDL_FUNC("TTF_GetFontKerningSizeGlyphs32", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("TTF_SetFontSDF", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_bool)
SDL_FUNC("TTF_GetFontSDF", ctypes.c_bool, ctypes.POINTER(TTF_Font))

TTF_SetError = SDL_GET_DLL(SDL_DLL).SDL_SetError
TTF_GetError = SDL_GET_DLL(SDL_DLL).SDL_GetError

TTF_Direction = ctypes.c_int

TTF_DIRECTION_LTR = 0
TTF_DIRECTION_RTL = 1
TTF_DIRECTION_TTB = 2
TTF_DIRECTION_BTT = 3

SDL_FUNC("TTF_SetFontDirection", ctypes.c_int, ctypes.POINTER(TTF_Font), TTF_Direction)
SDL_FUNC("TTF_SetFontScriptName", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p)
SDL_FUNC("TTF_SetFontLanguage", ctypes.c_int, ctypes.POINTER(TTF_Font), ctypes.c_char_p)
SDL_FUNC("TTF_IsFontScalable", ctypes.c_bool, ctypes.POINTER(TTF_Font))


import sys, os, inspect, ctypes, array

SDL_DLL, SDL_IMAGE_DLL, SDL_MIXER_DLL, SDL_NET_DLL, SDL_RTF_DLL, SDL_TTF_DLL = \
    "SDL3", "SDL3_image", "SDL3_mixer", "SDL3_net", "SDL3_rtf", "SDL3_ttf"

SDL_DLL_VAR_MAP = {}

for i in locals().copy():
    if i.startswith("SDL_") and i.endswith("_DLL"):
        SDL_DLL_VAR_MAP[i] = locals()[i]

SDL_DLL_VAR_MAP_INV = {v: k for k, v in SDL_DLL_VAR_MAP.items()}

if "PYSDL3_DISABLE_DOCS" not in os.environ:
    os.environ["PYSDL3_DISABLE_DOCS"] = "0"

__disable_docs__ = 0 < int(os.environ.get("PYSDL3_DISABLE_DOCS"))
__docs_file__ = os.path.join(os.path.dirname(__file__), "__docs__.py")

__initialized__ = __name__.split(".")[0] in sys.modules if "." in __name__ else False
__module__ = sys.modules[__name__.split(".")[0] if "." in __name__ else __name__]
__loader__ = ctypes.CDLL if "win32" not in sys.platform else ctypes.windll.LoadLibrary

if not __initialized__:
    functions, dllMap, dll = {}, {}, None
    binaryPath = os.path.join(os.path.dirname(__file__), "bin")

    for key, value in SDL_DLL_VAR_MAP.items():
        dllMap[value], functions[value] = \
            __loader__(os.path.join(binaryPath, ("{}.dll" if "win32" in sys.platform else "lib{}.so").format(value))), {}

else:
    functions, dllMap, dll, binaryPath = \
        __module__.functions, __module__.dllMap, __module__.dll, __module__.binaryPath

def SDL_DEREFERENCE(value):
    if isinstance(value, ctypes._Pointer):
        return value.contents
    
    elif hasattr(value, "_obj"):
        return value._obj
    
    else:
        return value

def SDL_FUNC_CACHE(func):
    cache = {}

    def wrapper(*args):
        if args in cache:
            return cache[args]
        
        else:
            cache[args] = func(*args)
            return cache[args]

    return wrapper

@SDL_FUNC_CACHE
def SDL_GET_DLL_NAME(dll):
    return {v: k for k, v in dllMap.items()}[dll]
        
def SDL_GET_DLL(name):
    return dllMap[name]

def SDL_SET_CURRENT_DLL(name):
    __module__.dll = SDL_GET_DLL(name)
    
def SDL_GET_CURRENT_DLL():
    return __module__.dll

def SDL_GET_FUNCTION_DLL(name):
    return __module__.functions[name].__dll__

def SDL_FUNC(name, retType, *args):
    dll = SDL_GET_CURRENT_DLL(); func = getattr(dll, name)
    func.__dll__, func.restype, func.argtypes = dll, retType, args
    if __disable_docs__: setattr(__module__, name, func)
    __module__.functions[SDL_GET_DLL_NAME(dll)][name] = func

from .SDL import *

if not __initialized__ and not __disable_docs__:
    result = "from .SDL import *\n\n"
    result += f"from .__init__ import ctypes, SDL_GET_DLL, \\\n"
    result += f"    {', '.join(list(SDL_DLL_VAR_MAP.keys()))}\n\n"
    types, definitions = set(), ""

    def getName(i):
        if i is None: return "None"
        if "CFunctionType" in i.__name__:
            return "ctypes._Pointer"

        if "LP_" in i.__name__ or not i.__name__.startswith("c_"):
            if "LP_" in i.__name__:
                types.add(i.__name__)

            return i.__name__
        
        else:
            return f"ctypes.{i.__name__}"

    for index, dll in enumerate(__module__.functions):
        if len(__module__.functions[dll]) == 0: continue
        definitions += f"# {dll}.dll implementation.\n\n"

        for _index, name in enumerate(__module__.functions[dll]):
            retType, args = \
                __module__.functions[dll][name].restype, __module__.functions[dll][name].argtypes

            definitions += f"def {name}({', '.join([f'_{index}: {getName(i)}' for index, i in enumerate(args)])}) -> {getName(retType)}:\n"
            definitions += f"    \"\"\"This function is auto-generated.\"\"\"\n"
            definitions += f"    return SDL_GET_DLL({SDL_DLL_VAR_MAP_INV[dll]}).{name}({', '.join([f'_{index}' for index, i in enumerate(args)])})"

            if _index != len(__module__.functions[dll]) - 1:
                definitions += "\n\n"

        if index != len(__module__.functions) - 1 and len(list(__module__.functions.values())[index + 1]) != 0:
            definitions += "\n\n"

    for i in types:
        count, name = i.count("LP_"), i.replace("LP_", "")
        result += f"class {i}({'ctypes.POINTER(' * count}{'ctypes.' if name.startswith('c_') else ''}{name}{')' * count}):\n"
        result += f"    \"\"\"This class is auto-generated.\"\"\"\n\n"

    with open(__docs_file__, "w") as file:
        file.write(result + definitions)

    from .__docs__ import *

else:
    if os.path.exists(__docs_file__):
        os.remove(__docs_file__)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_FALSE = 0
SDL_TRUE = 1

SDL_bool = ctypes.c_bool

Sint8 = ctypes.c_int8
SDL_MAX_SINT8 = 0x7F
SDL_MIN_SINT8 = ~0X7F

Uint8 = ctypes.c_uint8
SDL_MAX_UINT8 = 0xFF
SDL_MIN_UINT8 = 0X00

Sint16 = ctypes.c_int16
SDL_MAX_SINT16 = 0x7FFF
SDL_MIN_SINT16 = ~0X7FFF

Uint16 = ctypes.c_uint16
SDL_MAX_UINT16 = 0xFFFF
SDL_MIN_UINT16 = 0X0000

Sint32 = ctypes.c_int32
SDL_MAX_SINT32 = 0x7FFFFFFF
SDL_MIN_SINT32 = ~0X7FFFFFFF

Uint32 = ctypes.c_uint32
SDL_MAX_UINT32 = 0xFFFFFFFF
SDL_MIN_UINT32 = 0X00000000

Sint64 = ctypes.c_int64
SDL_MAX_SINT64 = 0x7FFFFFFFFFFFFFFF
SDL_MIN_SINT64 = ~0X7FFFFFFFFFFFFFFF

Uint64 = ctypes.c_uint64
SDL_MAX_UINT64 = 0xFFFFFFFFFFFFFFFF
SDL_MIN_UINT64 = 0X0000000000000000

SDL_Time = Sint64
SDL_MAX_TIME = SDL_MAX_SINT64
SDL_MIN_TIME = SDL_MIN_SINT64

SDL_FLT_EPSILON = 1.1920928955078125e-07

SDL_FunctionPointer = ctypes.CFUNCTYPE(None)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_MUTEX_TIMEDOUT = 1

class SDL_Mutex(ctypes.Structure):
    ...

SDL_FUNC("SDL_CreateMutex", ctypes.POINTER(SDL_Mutex))
SDL_FUNC("SDL_LockMutex", None, ctypes.POINTER(SDL_Mutex))
SDL_FUNC("SDL_TryLockMutex", ctypes.c_int, ctypes.POINTER(SDL_Mutex))
SDL_FUNC("SDL_UnlockMutex", None, ctypes.POINTER(SDL_Mutex))
SDL_FUNC("SDL_DestroyMutex", None, ctypes.POINTER(SDL_Mutex))

class SDL_RWLock(ctypes.Structure):
    ...

SDL_RWLOCK_TIMEDOUT = SDL_MUTEX_TIMEDOUT

SDL_FUNC("SDL_CreateRWLock", ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_LockRWLockForReading", None, ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_LockRWLockForWriting", None, ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_TryLockRWLockForReading", ctypes.c_int, ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_TryLockRWLockForWriting", ctypes.c_int, ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_UnlockRWLock", None, ctypes.POINTER(SDL_RWLock))
SDL_FUNC("SDL_DestroyRWLock", None, ctypes.POINTER(SDL_RWLock))

class SDL_Semaphore(ctypes.Structure):
    ...

SDL_FUNC("SDL_CreateSemaphore", ctypes.POINTER(SDL_Semaphore), ctypes.c_uint32)
SDL_FUNC("SDL_DestroySemaphore", None, ctypes.POINTER(SDL_Semaphore))
SDL_FUNC("SDL_WaitSemaphore", ctypes.c_int, ctypes.POINTER(SDL_Semaphore))
SDL_FUNC("SDL_TryWaitSemaphore", ctypes.c_int, ctypes.POINTER(SDL_Semaphore))
SDL_FUNC("SDL_WaitSemaphoreTimeout", ctypes.c_int, ctypes.POINTER(SDL_Semaphore), ctypes.c_int32)
SDL_FUNC("SDL_SignalSemaphore", ctypes.c_int, ctypes.POINTER(SDL_Semaphore))
SDL_FUNC("SDL_GetSemaphoreValue", ctypes.c_uint32, ctypes.POINTER(SDL_Semaphore))

class SDL_Condition(ctypes.Structure):
    ...

SDL_FUNC("SDL_CreateCondition", ctypes.POINTER(SDL_Condition))
SDL_FUNC("SDL_DestroyCondition", None, ctypes.POINTER(SDL_Condition))
SDL_FUNC("SDL_SignalCondition", ctypes.c_int, ctypes.POINTER(SDL_Condition))
SDL_FUNC("SDL_BroadcastCondition", ctypes.c_int, ctypes.POINTER(SDL_Condition))
SDL_FUNC("SDL_WaitCondition", ctypes.c_int, ctypes.POINTER(SDL_Condition), ctypes.POINTER(SDL_Mutex))
SDL_FUNC("SDL_WaitConditionTimeout", ctypes.c_int, ctypes.POINTER(SDL_Condition), ctypes.POINTER(SDL_Mutex), ctypes.c_int32)